Imports System.Threading.Tasks
Imports System.Windows
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports Windows.Storage
Imports WinPhoneFtp.FtpService

Namespace WinPhoneFtp.DemoApp

	Partial Public Class App
		Inherits Application

        Private PrivateClient As FtpClient
		Public Property Client As FtpClient
			Get
                Return PrivateClient
			End Get
			Private Set(ByVal value As FtpClient)
                PrivateClient = value
			End Set
		End Property

        Private PrivateServerCurrentWorkingDirectory As String
		Public Property ServerCurrentWorkingDirectory As String
			Get
                Return PrivateServerCurrentWorkingDirectory
			End Get
			Private Set(ByVal value As String)
                PrivateServerCurrentWorkingDirectory = value
			End Set
		End Property

		Public Property PhoneCurrentWorkingDirectory As Stack(Of StorageFolder)

		''' <summary>
		''' Provides easy access to the root frame of the Phone Application.
		''' </summary>
		''' <returns>The root frame of the Phone Application.</returns>
        Private Shared PrivateRootFrame As PhoneApplicationFrame
		Public Shared Property RootFrame As PhoneApplicationFrame
			Get
                Return PrivateRootFrame
			End Get
			Private Set(ByVal value As PhoneApplicationFrame)
                PrivateRootFrame = value
			End Set
		End Property

		''' <summary>
		''' Constructor for the Application object.
		''' </summary>
        Public Sub New()

            ' Global handler for uncaught exceptions.
            AddHandler UnhandledException, AddressOf Application_UnhandledException

            ' Standard XAML initialization
            InitializeComponent()

            ' Phone-specific initialization
            InitializePhoneApplication()

            ' Language display initialization
            'InitializeLanguage()

            ' Show graphics profiling information while debugging.
            If Debugger.IsAttached Then

                ' Display the current frame rate counters.
                'Application.Current.Host.Settings.EnableFrameRateCounter = True

                ' Show the areas of the app that are being redrawn in each frame.
                'Application.Current.Host.Settings.EnableRedrawRegions = True

                ' Enable non-production analysis visualization mode,
                ' which shows areas of a page that are handed off to GPU with a colored overlay.
                'Application.Current.Host.Settings.EnableCacheVisualization = True

                ' Prevent the screen from turning off while under the debugger by disabling
                ' the application's idle detection.
                ' Caution:- Use this under debug mode only. Application that disables user idle detection will continue to run
                ' and consume battery power when the user is not using the phone.
                PhoneApplicationService.Current.UserIdleDetectionMode = IdleDetectionMode.Disabled

            End If

        End Sub

		' Code to execute when the application is launching (eg, from Start)
		' This code will not execute when the application is reactivated
		Private Sub Application_Launching(ByVal sender As Object, ByVal e As LaunchingEventArgs)

			PhoneCurrentWorkingDirectory = New Stack(Of StorageFolder)
			PhoneCurrentWorkingDirectory.Push(Windows.Storage.ApplicationData.Current.LocalFolder)

		End Sub

		' Code to execute when the application is activated (brought to foreground)
		' This code will not execute when the application is first launched
		Private Sub Application_Activated(ByVal sender As Object, ByVal e As ActivatedEventArgs)

		End Sub

		' Code to execute when the application is deactivated (sent to background)
		' This code will not execute when the application is closing
		Private async Sub Application_Deactivated(ByVal sender As Object, ByVal e As DeactivatedEventArgs)

			If Client IsNot Nothing Then

                Await Client.DisconnectAsync
				Client = Nothing

			End If

		End Sub

		' Code to execute when the application is closing (eg, user hit Back)
		' This code will not execute when the application is deactivated
		Private async Sub Application_Closing(ByVal sender As Object, ByVal e As ClosingEventArgs)

			If Client IsNot Nothing Then

                Await Client.DisconnectAsync
				Client = Nothing

			End If

		End Sub

		' Code to execute if a navigation fails
		Private Sub RootFrame_NavigationFailed(ByVal sender As Object, ByVal e As NavigationFailedEventArgs)

			If Debugger.IsAttached Then

				' A navigation has failed; break into the debugger
				Debugger.Break()

			End If

		End Sub

		' Code to execute on Unhandled Exceptions
		Private Sub Application_UnhandledException(ByVal sender As Object, ByVal e As ApplicationUnhandledExceptionEventArgs)

			If Debugger.IsAttached Then

				' An unhandled exception has occurred; break into the debugger
				Debugger.Break()

			End If

		End Sub

		#Region "Phone application initialization"

		' Avoid double-initialization
		Private phoneApplicationInitialized As Boolean = False

		' Do not add any additional code to this method
		Private Sub InitializePhoneApplication()

			If phoneApplicationInitialized Then
                Exit Sub
			End If

			' Create the frame but don't set it as RootVisual yet; this allows the splash
			' screen to remain active until the application is ready to render.
			RootFrame = New PhoneApplicationFrame
			AddHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication

			' Handle navigation failures
			AddHandler RootFrame.NavigationFailed, AddressOf RootFrame_NavigationFailed

			' Handle reset requests for clearing the backstack
			AddHandler RootFrame.Navigated, AddressOf CheckForResetNavigation

			' Ensure we don't initialize again
			phoneApplicationInitialized = True

		End Sub

		' Do not add any additional code to this method
		Private Sub CompleteInitializePhoneApplication(ByVal sender As Object, ByVal e As NavigationEventArgs)

			' Set the root visual to allow the application to render
			If RootVisual IsNot RootFrame Then
				RootVisual = RootFrame
			End If

			' Remove this handler since it is no longer needed
			RemoveHandler RootFrame.Navigated, AddressOf CompleteInitializePhoneApplication

		End Sub

		Private Sub CheckForResetNavigation(ByVal sender As Object, ByVal e As NavigationEventArgs)

			' If the app has received a 'reset' navigation, then we need to check
			' on the next navigation to see if the page stack should be reset
			If e.NavigationMode = NavigationMode.Reset Then
				AddHandler RootFrame.Navigated, AddressOf ClearBackStackAfterReset
			End If

		End Sub

		Private Sub ClearBackStackAfterReset(ByVal sender As Object, ByVal e As NavigationEventArgs)

			' Unregister the event so it doesn't get called again
			RemoveHandler RootFrame.Navigated, AddressOf ClearBackStackAfterReset

			' Only clear the stack for 'new' (forward) and 'refresh' navigations
			If e.NavigationMode <> NavigationMode.New AndAlso e.NavigationMode <> NavigationMode.Refresh Then
                Exit Sub
			End If

			' For UI consistency, clear the entire page stack
            Do While RootFrame.RemoveBackEntry IsNot Nothing

                ' do nothing

            Loop

		End Sub

		#End Region

		Public async Function ConnectToFtpServer(ByVal FtpServerIpAddress As String, ByVal PortNumber As String) As Task

			If Client Is Nothing Then

				Client = New FtpClient(FtpServerIpAddress, PortNumber, RootFrame.Dispatcher)
				AddHandler Client.FtpAuthenticationFailed, AddressOf ftpClient_FtpAuthenticationFailed
				AddHandler Client.FtpAuthenticationSucceeded, AddressOf ftpClient_FtpAuthenticationSucceeded
				AddHandler Client.FtpConnected, AddressOf ftpClient_FtpConnected
				AddHandler Client.FtpDirectoryChangedFailed, AddressOf ftpClient_FtpDirectoryChangedFailed
				AddHandler Client.FtpDirectoryChangedSucceded, AddressOf ftpClient_FtpDirectoryChangedSucceded
				AddHandler Client.FtpDirectoryListed, AddressOf ftpClient_FtpDirectoryListed
				AddHandler Client.FtpDisconnected, AddressOf ftpClient_FtpDisconnected
				AddHandler Client.FtpFileDownloadFailed, AddressOf ftpClient_FtpFileDownloadFailed
				AddHandler Client.FtpFileDownloadSucceeded, AddressOf ftpClient_FtpFileDownloadSucceeded
				AddHandler Client.FtpFileTransferProgressed, AddressOf ftpClient_FtpFileTransferProgressed
				AddHandler Client.FtpFileUploadFailed, AddressOf ftpClient_FtpFileUploadFailed
				AddHandler Client.FtpFileUploadSucceeded, AddressOf ftpClient_FtpFileUploadSucceeded
				AddHandler Client.FtpPresentWorkingDirectoryReceived, AddressOf ftpClient_FtpPresentWorkingDirectoryReceived
                Await Client.ConnectAsync

			End If

		End Function

		Private Sub ftpClient_FtpPresentWorkingDirectoryReceived(ByVal sender As Object, ByVal e As FtpPresentWorkingDirectoryEventArgs)

			RootFrame.Dispatcher.BeginInvoke(Sub() ServerCurrentWorkingDirectory = e.PresentWorkingDirectory)

		End Sub

		Private Sub ftpClient_FtpFileUploadSucceeded(ByVal sender As Object, ByVal e As FtpFileTransferEventArgs)

		End Sub

		Private Sub ftpClient_FtpFileUploadFailed(ByVal sender As Object, ByVal e As FtpFileTransferFailedEventArgs)

		End Sub

		Private Sub ftpClient_FtpFileTransferProgressed(ByVal sender As Object, ByVal e As FtpFileTransferProgressedEventArgs)

		End Sub

		Private Sub ftpClient_FtpFileDownloadSucceeded(ByVal sender As Object, ByVal e As FtpFileTransferEventArgs)

		End Sub

		Private Sub ftpClient_FtpFileDownloadFailed(ByVal sender As Object, ByVal e As FtpFileTransferFailedEventArgs)

		End Sub

		Private Sub ftpClient_FtpDisconnected(ByVal sender As Object, ByVal e As FtpDisconnectedEventArgs)

			RootFrame.Dispatcher.BeginInvoke(Sub() MessageBox.Show(String.Format("You have been disconnected. Reason: {0}", e.DisconnectReason), "FTP Demo App", MessageBoxButton.OK))

		End Sub

		Private Sub ftpClient_FtpDirectoryListed(ByVal sender As Object, ByVal e As FtpDirectoryListedEventArgs)

		End Sub

		Private async Sub ftpClient_FtpDirectoryChangedSucceded(ByVal sender As Object, ByVal e As FtpDirectoryChangedEventArgs)

            Await Client.GetPresentWorkingDirectoryAsync

		End Sub

		Private Sub ftpClient_FtpDirectoryChangedFailed(ByVal sender As Object, ByVal e As FtpDirectoryChangedEventArgs)

		End Sub

		Private Sub ftpClient_FtpConnected(ByVal sender As Object, ByVal e As EventArgs)

			RootFrame.Navigate(New Uri("/Authenticate.xaml", UriKind.Relative))

		End Sub

		Private async Sub ftpClient_FtpAuthenticationSucceeded(ByVal sender As Object, ByVal e As EventArgs)

            Await Client.GetPresentWorkingDirectoryAsync

			RootFrame.Dispatcher.BeginInvoke(Sub() RootFrame.Navigate(New Uri("/FtpDirectoryBrowser.xaml", UriKind.Relative)))

		End Sub

		Private Sub ftpClient_FtpAuthenticationFailed(ByVal sender As Object, ByVal e As EventArgs)

			RootFrame.Dispatcher.BeginInvoke(Sub() MessageBox.Show("Login Unsuccessful", "FTP Demo App", MessageBoxButton.OK))

		End Sub

	End Class

End Namespace
