Imports System.Linq
Imports System.Net
Imports System.Windows
Imports System.IO
Imports System.Windows.Controls
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports Windows.Storage
Imports Windows.System.Display
Imports System.Windows.Threading

Namespace WinPhoneFtp.DemoApp

	Partial Public Class FileDownload
		Inherits PhoneApplicationPage

		Public Sub New()

			InitializeComponent()

		End Sub

		Private FileToDownload As String = String.Empty
		Private timer As DispatcherTimer = Nothing
		Private downloadedChunckSize As UInt32 = 0
		Private totalDownloadedSize As UInt32 = 0

		Protected Overrides Sub OnNavigatedTo(ByVal e As NavigationEventArgs)

			If Me.NavigationContext.QueryString.Count.Equals(1) AndAlso Me.NavigationContext.QueryString.ContainsKey("File") Then

				FileToDownload = Me.NavigationContext.QueryString("File")
				If (TryCast(App.Current, App)).ServerCurrentWorkingDirectory.Equals("/") Then

					txtblkDownloadFilename.Text = String.Format("File to download: /{0}", FileToDownload)

				Else

					txtblkDownloadFilename.Text = String.Format("File to download: {0}/{1}", (TryCast(App.Current, App)).ServerCurrentWorkingDirectory, FileToDownload)

				End If

			Else

				MessageBox.Show("Invalid Page Navigation detected", "FTP Demo App", MessageBoxButton.OK)
				Me.NavigationService.Navigate(New Uri("/FtpDirectoryBrowser.xaml", UriKind.Relative))

			End If
			MyBase.OnNavigatedTo(e)

		End Sub


		Protected Overrides Sub OnNavigatingFrom(ByVal e As NavigatingCancelEventArgs)

			RemoveHandler TryCast(App.Current, App).Client.FtpFileDownloadFailed, AddressOf Client_FtpFileDownloadFailed
			RemoveHandler TryCast(App.Current, App).Client.FtpFileDownloadSucceeded, AddressOf Client_FtpFileDownloadSucceeded
			RemoveHandler TryCast(App.Current, App).Client.FtpFileTransferProgressed, AddressOf Client_FtpFileTransferProgressed
			MyBase.OnNavigatingFrom(e)

		End Sub

		Private async Sub btnDownloadFile_Tap(ByVal sender As Object, ByVal e As System.Windows.Input.GestureEventArgs)

			btnDownloadFile.IsEnabled = False
			SystemTray.SetProgressIndicator(Me, New ProgressIndicator With {.IsIndeterminate = True, .Text = "Downloading file over FTP", .IsVisible = True})
			timer = New DispatcherTimer With {.Interval = TimeSpan.FromSeconds(1)}
			AddHandler timer.Tick, AddressOf timer_Tick
			AddHandler TryCast(App.Current, App).Client.FtpFileDownloadFailed, AddressOf Client_FtpFileDownloadFailed
			AddHandler TryCast(App.Current, App).Client.FtpFileDownloadSucceeded, AddressOf Client_FtpFileDownloadSucceeded
			AddHandler TryCast(App.Current, App).Client.FtpFileTransferProgressed, AddressOf Client_FtpFileTransferProgressed
			Dim localFile As StorageFile = await Windows.Storage.ApplicationData.Current.LocalFolder.CreateFileAsync(FileToDownload, CreationCollisionOption.ReplaceExisting)
            Await (TryCast(App.Current, App)).Client.DownloadFileAsync(Await localFile.OpenStreamForWriteAsync, FileToDownload)
			timer.Start()

		End Sub

		Private Sub timer_Tick(ByVal sender As Object, ByVal e As EventArgs)

			Me.downloadedChunckSize = 0
			If SystemTray.ProgressIndicator IsNot Nothing Then

				SystemTray.ProgressIndicator.Text = String.Format("{0} KB downloaded", Convert.ToSingle(totalDownloadedSize \ 1024).ToString("F2"))

			End If

		End Sub

		Private Sub Client_FtpFileTransferProgressed(ByVal sender As Object, ByVal e As FtpService.FtpFileTransferProgressedEventArgs)

            Me.downloadedChunckSize += e.BytesTransferred
			Me.totalDownloadedSize += e.BytesTransferred

		End Sub

		Private Sub Client_FtpFileDownloadSucceeded(ByVal sender As Object, ByVal e As FtpService.FtpFileTransferEventArgs)

			Me.Dispatcher.BeginInvoke(Sub()

				btnDownloadFile.IsEnabled = True
				SystemTray.ProgressIndicator = Nothing
				timer.Stop()
				MessageBox.Show("File downloaded successfully", "FTP Demo App", MessageBoxButton.OK)
				Me.NavigationService.Navigate(New Uri("/FtpDirectoryBrowser.xaml", UriKind.Relative))

			End Sub)

		End Sub

		Private Sub Client_FtpFileDownloadFailed(ByVal sender As Object, ByVal e As FtpService.FtpFileTransferFailedEventArgs)

			Me.Dispatcher.BeginInvoke(Sub()

				btnDownloadFile.IsEnabled = True
				SystemTray.ProgressIndicator = Nothing
				timer.Stop()
				MessageBox.Show(String.Format("Unable to download file. Reason: {0}", e.FileTransferFailReason), "FTP Demo App", MessageBoxButton.OK)
				Me.NavigationService.Navigate(New Uri("/FtpDirectoryBrowser.xaml", UriKind.Relative))

			End Sub)

		End Sub

		Private Sub appbarmnuDisconnect_Click(ByVal sender As Object, ByVal e As EventArgs)

			Me.NavigationService.Navigate(New Uri("/Disconnect.xaml", UriKind.Relative))

		End Sub

	End Class

End Namespace
