Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.IO
Imports System.Linq
Imports System.Runtime.InteropServices.WindowsRuntime
Imports System.Threading.Tasks
Imports WinPhoneFtp.FtpService
Imports Windows.Networking.Sockets
Imports Windows.Storage.Streams
Imports System.Windows.Threading

Namespace WinPhoneFtp.FtpService
    Public Class FtpClient
        Private Username As String = String.Empty
        Private Password As String = String.Empty
        Private FtpCommandSocket As TcpClientSocket = Nothing
        Private ftpCommand As FtpCommand = FtpCommand.None
        Private ftpPassiveOperation As FtpPassiveOperation = FtpPassiveOperation.None
        Private ftpFileInfo As FtpFileOperationInfo = Nothing
        Private FtpDataChannel As StreamSocket = Nothing
        Private fileListingData As List(Of Byte) = Nothing
        Private RemoteDirectory As String = String.Empty
        Private logger As Logger = Nothing
        Private UIDispatcher As Dispatcher = Nothing

        Public Event FtpConnected As EventHandler
        Public Event FtpAuthenticationSucceeded As EventHandler
        Public Event FtpAuthenticationFailed As EventHandler
        Public Event FtpDirectoryChangedSucceded As EventHandler(Of FtpDirectoryChangedEventArgs)
        Public Event FtpDirectoryChangedFailed As EventHandler(Of FtpDirectoryChangedEventArgs)
        Public Event FtpDisconnected As EventHandler(Of FtpDisconnectedEventArgs)
        Public Event FtpPresentWorkingDirectoryReceived As EventHandler(Of FtpPresentWorkingDirectoryEventArgs)
        Public Event FtpFileUploadSucceeded As EventHandler(Of FtpFileTransferEventArgs)
        Public Event FtpFileUploadFailed As EventHandler(Of FtpFileTransferFailedEventArgs)
        Public Event FtpFileDownloadSucceeded As EventHandler(Of FtpFileTransferEventArgs)
        Public Event FtpFileDownloadFailed As EventHandler(Of FtpFileTransferFailedEventArgs)
        Public Event FtpDirectoryListed As EventHandler(Of FtpDirectoryListedEventArgs)
        Public Event FtpFileTransferProgressed As EventHandler(Of FtpFileTransferProgressedEventArgs)

        Public Sub New(ByVal FtpServerIpAddress As String, ByVal UIDispatcher As Dispatcher)
            Me.New(FtpServerIpAddress, "21", UIDispatcher)
        End Sub

        Public Sub New(ByVal FtpServerIpAddress As String, ByVal PortNumber As String, ByVal UIDispatcher As Dispatcher)
            Me.UIDispatcher = UIDispatcher
            logger = Logger.GetDefault(UIDispatcher)
            logger.AddLog(String.Format("FTP Server IP Address: {0} with port {1}", FtpServerIpAddress, PortNumber))
            FtpCommandSocket = New TcpClientSocket(FtpServerIpAddress, PortNumber, 512, "FTP Command Channel", UIDispatcher)
            AddHandler FtpCommandSocket.DataReceived, AddressOf FtpClientSocket_DataReceived
            AddHandler FtpCommandSocket.ErrorOccured, AddressOf FtpClientSocket_ErrorOccured
            AddHandler FtpCommandSocket.SocketClosed, AddressOf FtpClientSocket_SocketClosed
            AddHandler FtpCommandSocket.SocketConnected, AddressOf FtpCommandSocket_SocketConnected
            IsConnected = False
        End Sub

        Private PrivateIsConnected As Boolean
        Public Property IsConnected As Boolean
            Get
                Return PrivateIsConnected
            End Get
            Private Set(ByVal value As Boolean)
                PrivateIsConnected = value
            End Set
        End Property

        Private PrivateIsBusy As Boolean
        Public Property IsBusy As Boolean
            Get
                Return PrivateIsBusy
            End Get
            Private Set(ByVal value As Boolean)
                PrivateIsBusy = value
            End Set
        End Property

        Public Async Function ConnectAsync() As Task
            If Not IsConnected Then
                logger.AddLog("FTP Command Channel Initailized")
                Await FtpCommandSocket.PrepareSocketAsync
            End If
        End Function

        Private Async Sub FtpClientSocket_DataReceived(ByVal sender As Object, ByVal e As DataReceivedEventArgs)
            Dim Response As String = System.Text.Encoding.UTF8.GetString(e.GetData(), 0, e.GetData().Length)
            logger.AddLog(String.Format("FTPServer -> {0}", Response))
            Select Case ftpPassiveOperation
                Case FtpPassiveOperation.FileDownload
                    ftpCommand = FtpCommand.None
                    If Response.StartsWith("150") OrElse Response.StartsWith("125") Then
                        IsBusy = True
                        Dim dataReader As New DataReader(FtpDataChannel.InputStream)
                        dataReader.InputStreamOptions = InputStreamOptions.Partial
                        Do While Not (Await dataReader.LoadAsync(32768)).Equals(0)
                            Dim databuffer As IBuffer = dataReader.DetachBuffer
                            RaiseFtpFileTransferProgressedEvent(databuffer.Length, False)
                            Await ftpFileInfo.LocalFileStream.WriteAsync(databuffer.ToArray, 0, Convert.ToInt32(databuffer.Length))
                        Loop
                        Await ftpFileInfo.LocalFileStream.FlushAsync
                        dataReader.Dispose()
                        dataReader = Nothing
                        'FtpDataChannel.Dispose()
                        'FtpDataChannel = Nothing
                        RaiseFtpFileDownloadSucceededEvent(ftpFileInfo.LocalFileStream, ftpFileInfo.RemoteFile)
                    ElseIf Response.StartsWith("226") Then
                        IsBusy = False
                        ftpPassiveOperation = FtpPassiveOperation.None
                    Else
                        IsBusy = False
                        ftpPassiveOperation = FtpPassiveOperation.None
                        RaiseFtpFileDownloadFailedEvent(ftpFileInfo.LocalFileStream, ftpFileInfo.RemoteFile, FtpFileTransferFailureReason.FileDoesNotExist)
                    End If

                Case FtpPassiveOperation.FileUpload
                    ftpCommand = FtpCommand.None
                    If Response.StartsWith("150") OrElse Response.StartsWith("125") Then
                        IsBusy = True
                        Dim dataWriter As New DataWriter(FtpDataChannel.OutputStream)
                        Dim data(32767) As Byte
                        Do While Not (Await ftpFileInfo.LocalFileStream.ReadAsync(data, 0, data.Length)).Equals(0)
                            dataWriter.WriteBytes(data)
                            Await dataWriter.StoreAsync
                            RaiseFtpFileTransferProgressedEvent(Convert.ToUInt32(data.Length), True)
                        Loop
                        Await dataWriter.FlushAsync
                        dataWriter.Dispose()
                        dataWriter = Nothing
                        FtpDataChannel.Dispose()
                        FtpDataChannel = Nothing
                    ElseIf Response.StartsWith("226") Then
                        IsBusy = False
                        ftpPassiveOperation = FtpPassiveOperation.None
                        RaiseFtpFileUploadSucceededEvent(ftpFileInfo.LocalFileStream, ftpFileInfo.RemoteFile)
                        ftpFileInfo = Nothing
                    Else
                        IsBusy = False
                        ftpPassiveOperation = FtpPassiveOperation.None
                        RaiseFtpFileUploadFailedEvent(ftpFileInfo.LocalFileStream, ftpFileInfo.RemoteFile, FtpFileTransferFailureReason.FileDoesNotExist)
                        ftpFileInfo = Nothing
                    End If

                Case FtpPassiveOperation.ListDirectory
                    ftpCommand = FtpCommand.None
                    If Response.StartsWith("150") OrElse Response.StartsWith("125") Then
                        IsBusy = True
                        Dim dataReader As New DataReader(FtpDataChannel.InputStream)
                        dataReader.InputStreamOptions = InputStreamOptions.Partial
                        fileListingData = New List(Of Byte)
                        Do While Not (Await dataReader.LoadAsync(1024)).Equals(0)
                            fileListingData.AddRange(dataReader.DetachBuffer.ToArray)
                        Loop
                        dataReader.Dispose()
                        dataReader = Nothing
                        FtpDataChannel.Dispose()
                        FtpDataChannel = Nothing
                        Dim listingData As String = System.Text.Encoding.UTF8.GetString(fileListingData.ToArray, 0, fileListingData.ToArray.Length)
                        Dim listings() As String = listingData.Split(New String() {Constants.vbCr, Constants.vbCrLf}, StringSplitOptions.RemoveEmptyEntries)
                        Dim Filenames As New List(Of String)
                        Dim Directories As New List(Of String)
                        For Each listing As String In listings
                            If listing.StartsWith("drwx") OrElse listing.Contains("<DIR>") Then
                                Directories.Add(listing.Split(New Char() {" "c}).Last)
                            Else
                                Filenames.Add(listing.Split(New Char() {" "c}).Last)
                            End If
                        Next listing
                        RaiseFtpDirectoryListedEvent(Directories.ToArray, Filenames.ToArray)
                    ElseIf Response.StartsWith("226") Then
                        IsBusy = False
                        ftpFileInfo = Nothing
                        ftpPassiveOperation = FtpPassiveOperation.None
                    End If

                Case FtpPassiveOperation.None
                    Select Case ftpCommand
                        Case FtpCommand.Username
                            If Response.StartsWith("501") Then
                                IsBusy = False
                                RaiseFtpAuthenticationFailedEvent()
                                Exit Select
                            End If
                            Me.ftpCommand = FtpCommand.Password
                            logger.AddLog(String.Format("FTPClient -> PASS {0}" & vbCrLf, Me.Password))
                            Await FtpCommandSocket.SendDataAsync(String.Format("PASS {0}" & vbCrLf, Me.Password))

                        Case FtpCommand.Password
                            Me.ftpCommand = FtpCommand.None
                            IsBusy = False
                            If Response.Contains("530") Then
                                RaiseFtpAuthenticationFailedEvent()
                            Else
                                RaiseFtpAuthenticationSucceededEvent()
                            End If

                        Case FtpCommand.ChangeWorkingDirectory
                            IsBusy = False
                            If Response.StartsWith("550") Then
                                RaiseFtpDirectoryChangedFailedEvent(Me.RemoteDirectory)
                            Else
                                RaiseFtpDirectoryChangedSuccededEvent(Me.RemoteDirectory)
                            End If

                        Case FtpCommand.PresentWorkingDirectory
                            If Response.StartsWith("257") Then
                                IsBusy = False
                                RaiseFtpPresentWorkingDirectoryReceivedEvent(Response.Split(New Char() {" "c, """"c}, StringSplitOptions.RemoveEmptyEntries)(1))
                            End If

                        Case FtpCommand.Type
                            ftpCommand = FtpCommand.Passive
                            logger.AddLog("FTPClient -> PASV" & vbCrLf)
                            Await FtpCommandSocket.SendDataAsync("PASV" & vbCrLf)

                        Case FtpCommand.Passive
                            If Response.StartsWith("227") Then
                                Await PrepareDataChannelAsync(Response)
                                If ftpFileInfo IsNot Nothing Then
                                    If ftpFileInfo.IsUpload Then
                                        ftpPassiveOperation = FtpPassiveOperation.FileUpload
                                        logger.AddLog(String.Format("FTPClient -> STOR {0}" & vbCrLf, ftpFileInfo.RemoteFile))
                                        Await FtpCommandSocket.SendDataAsync(String.Format("STOR {0}" & vbCrLf, ftpFileInfo.RemoteFile))
                                    Else
                                        ftpPassiveOperation = FtpPassiveOperation.FileDownload
                                        logger.AddLog(String.Format("FTPClient -> RETR {0}" & vbCrLf, ftpFileInfo.RemoteFile))
                                        Await FtpCommandSocket.SendDataAsync(String.Format("RETR {0}" & vbCrLf, ftpFileInfo.RemoteFile))
                                    End If
                                Else
                                    fileListingData = New List(Of Byte)
                                    ftpPassiveOperation = FtpPassiveOperation.ListDirectory
                                    logger.AddLog("FTPClient -> LIST" & vbCrLf)
                                    Await FtpCommandSocket.SendDataAsync("LIST" & vbCrLf)
                                End If
                            End If

                        Case FtpCommand.Logout
                            ftpCommand = FtpCommand.None

                        Case FtpCommand.None
                    End Select
            End Select

        End Sub

        Private Sub FtpCommandSocket_SocketConnected(ByVal sender As Object, ByVal e As EventArgs)
            IsConnected = True
            RaiseFtpConnectedEvent()
        End Sub

        Private Sub FtpClientSocket_ErrorOccured(ByVal sender As Object, ByVal e As ErrorOccuredEventArgs)
            logger.AddLog(e.ExceptionObject.Message)
            IsConnected = False
            RaiseFtpDisconnectedEvent(FtpDisconnectReason.SocketError)
        End Sub

        Private Sub FtpClientSocket_SocketClosed(ByVal sender As Object, ByVal e As SocketClosedEventArgs)
            IsConnected = False
            If Not ftpCommand.Equals(FtpCommand.Logout) Then
                RaiseFtpDisconnectedEvent(FtpDisconnectReason.SocketClosed)
            Else
                RaiseFtpDisconnectedEvent(FtpDisconnectReason.QuitCommand)
            End If
        End Sub

        Public Async Function AuthenticateAsync() As Task
            Await AuthenticateAsync("anonymous", "m@m.com") 'TODO: test value only
        End Function

        Public Async Function AuthenticateAsync(ByVal Username As String, ByVal Password As String) As Task
            ftpCommand = FtpCommand.Username
            Me.Username = Username
            Me.Password = Password
            logger.AddLog(String.Format("FTPClient -> USER {0}" & vbCrLf, Username))
            Await FtpCommandSocket.SendDataAsync(String.Format("USER {0}" & vbCrLf, Username))
        End Function

        Public Async Function ChangeWorkingDirectoryAsync(ByVal RemoteDirectory As String) As Task
            If Not IsBusy Then
                Me.RemoteDirectory = RemoteDirectory
                ftpCommand = FtpCommand.ChangeWorkingDirectory
                logger.AddLog(String.Format("FTPClient -> CWD {0}" & vbCrLf, RemoteDirectory))
                Await FtpCommandSocket.SendDataAsync(String.Format("CWD {0}" & vbCrLf, RemoteDirectory))
            End If
        End Function

        Public Async Function GetPresentWorkingDirectoryAsync() As Task
            If Not IsBusy Then
                ftpCommand = FtpCommand.PresentWorkingDirectory
                logger.AddLog("FTPClient -> PWD" & vbCrLf)
                Await FtpCommandSocket.SendDataAsync("PWD" & vbCrLf)
            End If
        End Function

        Public Async Function GetDirectoryListingAsync() As Task
            If Not IsBusy Then
                fileListingData = Nothing
                ftpFileInfo = Nothing
                IsBusy = True
                ftpCommand = FtpCommand.Passive
                logger.AddLog("FTPClient -> PASV" & vbCrLf)
                Await FtpCommandSocket.SendDataAsync("PASV" & vbCrLf)
            End If
        End Function

        Public Async Function UploadFileAsync(ByVal LocalFileStream As System.IO.Stream, ByVal RemoteFilename As String) As Task
            If Not IsBusy Then
                ftpFileInfo = Nothing
                IsBusy = True
                ftpFileInfo = New FtpFileOperationInfo(LocalFileStream, RemoteFilename, True)
                ftpCommand = FtpCommand.Type
                logger.AddLog("FTPClient -> TYPE I" & vbCrLf)
                Await FtpCommandSocket.SendDataAsync("TYPE I" & vbCrLf)
            End If
        End Function

        Public Async Function DownloadFileAsync(ByVal LocalFileStream As System.IO.Stream, ByVal RemoteFilename As String) As Task
            If Not IsBusy Then
                ftpFileInfo = Nothing
                IsBusy = True
                ftpFileInfo = New FtpFileOperationInfo(LocalFileStream, RemoteFilename, False)
                ftpCommand = FtpCommand.Type
                logger.AddLog("FTPClient -> TYPE I" & vbCrLf)
                Await FtpCommandSocket.SendDataAsync("TYPE I" & vbCrLf)
            End If
        End Function

        Public Async Function DisconnectAsync() As Task
            ftpCommand = FtpCommand.Logout
            logger.AddLog("FTPClient -> QUIT" & vbCrLf)
            Await FtpCommandSocket.SendDataAsync("QUIT" & vbCrLf)
        End Function

        Private Async Function PrepareDataChannelAsync(ByVal ChannelInfo As String) As Task
            ChannelInfo = ChannelInfo.Remove(0, "227 Entering Passive Mode".Length) 'TODO: test value only
            'Configure the IP Address
            Dim Splits() As String = ChannelInfo.Substring(ChannelInfo.IndexOf("(") + 1, ChannelInfo.Length - ChannelInfo.IndexOf("(") - 5).Split(New Char() {","c, " "c}, StringSplitOptions.RemoveEmptyEntries)
            Dim Ipaddr As String = String.Join(".", Splits, 0, 4)

            'Calculate the Data Port
            Dim port As Int32 = Convert.ToInt32(Splits(4))
            port = ((port << 8) Or Convert.ToInt32(Splits(5)))
            logger.AddLog(String.Format("FTP Data Channel IPAddress: {0}, Port: {1}", Ipaddr, port))
            FtpDataChannel = New StreamSocket
            Await FtpDataChannel.ConnectAsync(New Windows.Networking.HostName(Ipaddr), port.ToString)
            logger.AddLog("FTP Data Channel connected")
        End Function

        Private Sub RaiseFtpAuthenticationSucceededEvent()
            RaiseEvent FtpAuthenticationSucceeded(Me, EventArgs.Empty)
        End Sub

        Private Sub RaiseFtpAuthenticationFailedEvent()
            RaiseEvent FtpAuthenticationFailed(Me, EventArgs.Empty)
        End Sub

        Private Sub RaiseFtpDirectoryChangedSuccededEvent(ByVal RemoteDirectory As String)
            RaiseEvent FtpDirectoryChangedSucceded(Me, New FtpDirectoryChangedEventArgs(RemoteDirectory))
        End Sub

        Private Sub RaiseFtpDirectoryChangedFailedEvent(ByVal RemoteDirectory As String)
            RaiseEvent FtpDirectoryChangedFailed(Me, New FtpDirectoryChangedEventArgs(RemoteDirectory))
        End Sub

        Private Sub RaiseFtpPresentWorkingDirectoryReceivedEvent(ByVal PresentWorkingDirectory As String)
            RaiseEvent FtpPresentWorkingDirectoryReceived(Me, New FtpPresentWorkingDirectoryEventArgs(PresentWorkingDirectory))
        End Sub

        Private Sub RaiseFtpFileUploadSucceededEvent(ByVal LocalFileStream As Stream, ByVal RemoteFile As String)
            RaiseEvent FtpFileUploadSucceeded(Me, New FtpFileTransferEventArgs(LocalFileStream, RemoteFile))
        End Sub

        Private Sub RaiseFtpFileUploadFailedEvent(ByVal LocalFileStream As Stream, ByVal RemoteFile As String, ByVal FileTransferFailReason As FtpFileTransferFailureReason)
            RaiseEvent FtpFileUploadFailed(Me, New FtpFileTransferFailedEventArgs(LocalFileStream, RemoteFile, FileTransferFailReason))
        End Sub

        Private Sub RaiseFtpFileDownloadSucceededEvent(ByVal LocalFileStream As Stream, ByVal RemoteFile As String)
            RaiseEvent FtpFileDownloadSucceeded(Me, New FtpFileTransferEventArgs(LocalFileStream, RemoteFile))
        End Sub

        Private Sub RaiseFtpFileDownloadFailedEvent(ByVal LocalFileStream As Stream, ByVal RemoteFile As String, ByVal FileTransferFailReason As FtpFileTransferFailureReason)
            RaiseEvent FtpFileDownloadFailed(Me, New FtpFileTransferFailedEventArgs(LocalFileStream, RemoteFile, FileTransferFailReason))
        End Sub

        Private Sub RaiseFtpConnectedEvent()
            RaiseEvent FtpConnected(Me, EventArgs.Empty)
        End Sub

        Private Sub RaiseFtpDisconnectedEvent(ByVal DisconnectReason As FtpDisconnectReason)
            RaiseEvent FtpDisconnected(Me, New FtpDisconnectedEventArgs(DisconnectReason))
        End Sub

        Private Sub RaiseFtpDirectoryListedEvent(ByVal Directories() As String, ByVal Filenames() As String)
            RaiseEvent FtpDirectoryListed(Me, New FtpDirectoryListedEventArgs(Directories, Filenames))
        End Sub

        Private Sub RaiseFtpFileTransferProgressedEvent(ByVal BytesTransfered As UInt32, ByVal IsUpload As Boolean)
            RaiseEvent FtpFileTransferProgressed(Me, New FtpFileTransferProgressedEventArgs(BytesTransfered, IsUpload))
        End Sub
    End Class
End Namespace